﻿/*
---------------------------------------------------------------------------
 $Revision: 1.1.2.14.2.1 $
 
 $Date: 2018/07/02 12:04:05 $
 
 NAME: sea_export_complete
 
 TYPE: macro
 
 DESCRIPTION:
   This macro does the following
    - reads in UI fields from JSON file
    - reads export table information from the export directory
    - depending on the options 
 
 SAS VERSION:
 9.4
 
 PARAMETERS:

 From the Export UI:
 libraryname = Required, Name of library, must be unique
 libref = Required, SAS unique libref
 username= Required, name of user 
 library_desc= Optional, description of exported data
 
 REQUIREMENT: Macro is run within a SAS session that is connected to the
              OMR
              For example - SAS workspace server or SAS stored process server
 
 HISTORY:
 userid yyyymmdd defectid desc
 cabahl 20171030          Initial
-----------------------------------------------------------------------------
 Copyright (c) 2017, SAS Institute Inc., Cary, NC, USA, All Rights Reserved
------------------------------------------------------------------------------
*/

%macro sea_export_complete;

    %global retcode sea_status_type SEA_RUN_MESSAGES_DS  encrypt seaexporterror;
    %local currentMacro
           ;
    %let retcode=0;
    %let delete_export=0;
    %let file_handling=;
    %let errortype=;

    %let currentMacro=&sysmacroname;
    %let SEA_RUN_MESSAGES_DS=work.meta_run_messages_ds;
    %let sea_run_messages_ds=work.meta_run_messages_ds;

    proc printto log="&haf_output_folder/EXPORT_COMPLETE.log" new;
    run;
  
    %let haf_macro_nm=_NONE_;
    %sea_adm_allocate_libraries;

    libname eid json "&haf_json_input_file";

    data null_;
      set eid.alldata;
      if upcase(p1)='EXPORTTYPE' then do;
         value=upcase(value);
         call symput(upcase(p1),value);
      end;
      if upcase(p1)='EXPORTID' then do;      
        call symput('dsn',kcompress('_'||value||'_constants'));
      end;  
    run;

    libname export "&haf_output_folder";
    %if &EXPORTTYPE ne RWE %then %do;   
      libname outgo  "&haf_input_folder";

      %let clibref=outgo;      
    %end;
    %else %let clibref=export;
    
    /* Step 1 - check parms */
    /* Determine if all export files created */
    %if %sysfunc(exist(export.meta_ex_run_messages_dataset))>0 %then %do;
      data error_subset_init
           error_subset_grid
           error_subset_term
        ;
        set export.meta_ex_run_messages_dataset;
        if message_type_txt in ('WARNING' 'NOTE') then status='S';
          else status='F'; 

        if status='F' then do;
          if macro_name='sea_MAIN_SUBSET_DATA_INIT' then output error_subset_init;
          if macro_name='sea_MAIN_SUBSET_DATA_MAIN' then output error_subset_grid;
          if macro_name='sea_MAIN_SUBSET_DATA_TERM' then output error_subset_term;
        end;
      run;
    %end;
    %else %do;
      data error_subset_init
           error_subset_grid
           error_subset_term
        ;
        length status $1;
        stop;
      run;
    %end;

   /*************** LOG CHECK ***************/
   filename logs "&haf_output_folder";
   data logs(keep=v file);
     d=dopen("logs");
     v=0;
     do i = 1 to dnum(d);
      file = dread(d,i);
      if kscan(file,2,'.')='log' then
        do;
        v=v+1;
        call symput(kcompress("log"||put(v,2.)),kstrip(file));
        output; 
      end;
    end;
    call symput('logs',put(v,2.));
    d=dclose(d);
   run;
 
   %do i = 1 %to &logs; 
      %let SEA_ERROR_FLG=;
      filename a "&haf_output_folder./&&log&i";
      data _null_;
         length errchk $500.;
         format errchk $500.;
         infile a missover delimiter='@';
         input errchk $;
 
         if (klength(trim(left(errchk))) ge 5 and ksubstr(trim(left(errchk)),1,5)="ERROR") then
             do;
                 call symputx('SEA_ERROR_FLG','Y');
                 stop;
             end;
      run;    

      data logs;
        set logs;
        if v=&i then sea_error_flg="&sea_error_flg";
      run;
   %end;

   /*************** LOG CHECK ***************/

    /* check for library existence */
    %if %sysfunc(exist(&clibref..&dsn))=0 %then %do;
       %let retcode=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_DATASET_NOTEXIST_ERROR,noquote,&dsn));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;

    data _constants(drop=libname)
         tables(keep=memname);
      length group $4 p1 $32 value path omr_folder $1000;
      set &clibref..&dsn (in=a)     
          sashelp.vstable(where=(libname='EXPORT' and memname not like 'META_%')
                          keep=memname libname 
                          in=c)
          end=eof;
      retain m t v 0;
      p1=upcase(p1);
      if a and required=1 then do;
        if kcompress(value)='' 
           or kindex(value,'<')>0 
           then do;
           m=m+1;
           call symput(kcompress('missing_'||put(m,2.)),p1);
        end;  
      end;

      if a then do;
         group='AUI';
         if p1='HAF_EMAIL' then email_order=1;
         if p1='EXPORTNAME' then email_order=2;
         if p1='EXPORTDESCRIPTION' then email_order=3;
         if p1='EXPORT_DIRECTORY' then email_order=4;
         call symput(p1,kstrip(value));
      end;
      if c then do;
        group='TBL';
        dirpath="&haf_output_folder";
        path=kcompress(dirpath||'/'||lowcase(memname)||'.sas7bdat');
        t=t+1;                      
      end;

      if group ne '' then output _constants;
      if group = 'TBL' then output tables;

      if eof then do; 
         call symput('create_dttm',put(datetime(),datetime.));
         call symput('missing_parms',put(m,2.));
         call symput('tables',kcompress(put(t,best.)));
         call symput('missing_cols',put(v,2.));
      end;  
    run;

    %if %symexist(LIBRARYNAME)=0 %then %let LIBRARYNAME=;

    /* Error out if subsetting process failed */
    %if %sysfunc(fileexist(&haf_output_folder./export_subset_ok.txt))=0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_EXPORT_SUBSET_TERM_ERROR,noquote,&HAF_OUTPUT_FOLDER));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;   
    
    %if %sea_utl_get_observation_count(indsn=error_subset_init) > 0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_EXPORT_SUBSET_INIT_ERROR,noquote));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;

    %if %sea_utl_get_observation_count(indsn=error_subset_grid) > 0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_EXPORT_SUBSET_GRID_ERROR,noquote));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;

    %if %sea_utl_get_observation_count(indsn=error_subset_term) > 0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_EXPORT_SUBSET_TERM_ERROR,noquote,&HAF_OUTPUT_FOLDER));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;                

    data log_check;
      set logs end=eof;
      retain errors 0;
      if sea_error_flg ne '' then errors=errors+1;
      if eof then call symput('log_errors',kcompress(put(errors,best.)));
    run;
 
 
    %if &log_errors > 0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_EXPORT_SUBSET_TERM_ERROR,noquote,&HAF_OUTPUT_FOLDER));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;   
  
    /* Error out if expected parms missing */
    %if &missing_parms > 0 %then %do; 
      %let miss_parms=; 
      %do m=1 %to &missing_parms;
         %let miss_parms=&miss_parms &&missing_&m; 
         %let retcode=1;
         %sea_error(msg1=%sysfunc(sasmsg(&sea_message_ds,_parm_missing_error,noquote,&&missing_&m)));
      %end;

      %let delete_export=1;
      %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_PARM_MISSING,noquote,&miss_parms));
      %GOTO EXIT;

    %end;

    /* Error out if no tables */
    %if &tables=0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_NO_DATA_OMR_ERROR,noquote,&HAF_OUTPUT_FOLDER));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;

    proc sql;
      create table export_spec as
        select memname as export_table_nm
              ,upcase(name) as export_column_nm 
              ,'DS' length=4 as export_storage_type_cd 
          from dictionary.columns
         where libname='EXPORT'
           and memname in (select memname from tables where memname not like "META%");
    quit;

    %if %sea_utl_get_observation_count(indsn=export.export_spec)=0 %then %do;
       %let retcode=1;
       %let delete_export=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_NO_DATA_OMR_ERROR,noquote,&HAF_OUTPUT_FOLDER));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;
    %end;    
    
    /* Step 3 - determine export type, process accordingly */
    %if &exportType=FS %then %do;
      /* placeholder - no additional handling needed */
    %end;

    %if &exporttype=ARCHIVE %then %do;

      %sea_export_archive(inlib=export
                         ,tabledsn=tables);

      %if &retcode ne 0 %then %do;
        %let retcode=1;
        %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_ARCHIVE_001_ERROR,noquote));
        %let file_handling=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_FILE_HANDLING_ARCHIVE_NOTE,noquote));
        %let errortype=ARCHIVE;
        %GOTO EXIT;
      %end;

    %end;

    %let omr_parms=0;
    %if &exporttype=OMR or &exporttype=LASR %then %do;

       data tables;
        set tables end=eof;
        where kscan(memname,1,'_') ne 'META';
       run;  

       %sea_export_omr_table_register(dirpath=&haf_output_folder
                                    ,library_name=&libraryname);

       proc sql noprint;
         select count(*) as omr_parms
           into :omr_parms
           from _constants
          where p1 like 'OMR_%'; 
       quit;

       %if &retcode=1 or &omr_parms < 2 %then %do;
          %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_OMR_ERROR,noquote));
          %let file_handling=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_FILE_HANDLING_OMRBAD_NOTE,noquote));
          %let errortype=OMR;
       %end;

       %let OMR_FOLDER=;
       data _constants;
         set _constants;
         if p1='OMR_FOLDER' then do;
           call symput(p1,kstrip(value));
           email_order=1;
         end;
         if p1='OMR_LIB_ID' then call symput(p1,kstrip(value));
       run;
       %put &omr_folder;
    %end; 


   %if &retcode = 0 %then %do;
     /* Step Update SEA_EXPORT - Audit record */
     /* Get total disk space used by all export output files in megabytes */
     data _constants(drop=rc i diskspace dir fid did tot_diskspace files file);
       set _constants end=eof;
       if p1 =: 'OMR' or p1 in ('EXPORTID' 'TOTAL_INMEMSIZE') then updt_flg=1;
       output;
       if eof then do;
         dir="&haf_output_folder";
         rc=filename('exp',dir);
         did=dopen('exp');
         tot_diskspace=0;
         if did > 0 then do;
           group='FLSZ';

           files=dnum(did);
           call symput('files',put(files,4.));
           length file $500;
           if files > 0 then do;
             do i=1 to files;
               file=kstrip(dir)||"/"||dread(did,i);
               rc=filename('fx',file);
               fid=fopen('fx');
               diskspace=input(finfo(fid,"File Size (bytes)"),8.);
               tot_diskspace=tot_diskspace+diskspace;
               rc=fclose(fid);
             end;
           end;
         end;
         rc=dclose(did);
         p1='TOTAL_DISKSPACE';
         value=kcompress(put(round((tot_diskspace/1024/1024),1),best.));
         call symput(p1,value);
         updt_flg=1;
         output;
       end;
     run;

     %if &files=0 %then %do;
        %let retcode=1;
        %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_NO_DATA_OMR_ERROR,noquote,&HAF_INPUT_FOLDER));
        %sea_error(msg1=&seaexporterror);
        %GOTO EXIT;
     %end;
       
     /* Update SEA_EXPORT */
     proc sql;
       update &oralib..sea_export 
         set export_dir_path="&haf_output_folder"
            ,file_disk_size=&total_diskspace
            ,export_status="CREATED"
            ,export_updt_dttm=datetime()
           %if %symexist(OMR_FOLDER)>0 %then %do;
            ,omr_path="&omr_folder"
           %end;   
           %if %symexist(OMR_LIB_ID)>0 %then %do;  
            ,omr_lib_id="&omr_lib_id"
           %end;
       where export_id="&exportid";
     run; 

     proc sql noprint;
       select count(*) as cnt
         into :audit
         from &oralib..sea_export
        where export_id="&exportid" and export_status="CREATED";
     quit;

     %if %SYMEXIST(AUDIT)=0 %then %do;
       %let retcode=1;
       %let seaexporterror=%sysfunc(sasmsg(&sea_message_ds,_sea_EXPORT_AUDIT_UPDT_ERROR,noquote));
       %sea_error(msg1=&seaexporterror);
       %GOTO EXIT;       
     %end;

     data export.export_spec;
       length export_id $36
              export_table_nm
              export_column_nm $40
              export_storage_type_cd $4
              ;
       set export_spec (in=a)     
       ;
       export_id = "&exportid";
     run; 
     
     proc sql noprint;
       %let rowcnt=0;
       select count(*) as cnt
         into :rowcnt
         from &oralib..sea_export_spec
        where export_id="&exportid";
     
       %if &rowcnt=0 %then %do;
        delete from &oralib..sea_export_spec
         where export_id="&exportid";
       %end;  
     quit;   
    
     proc append base=&oralib..sea_export_spec
                 data=export.export_spec;
     run;

   %end;

   %if &exporttype=RWE %then %do;
     %rwe_db_connect_info(dblib_ext=rwe_app_lib);

     %let membercount=%sea_utl_get_observation_count(indsn=export.rwe_member);

     /* Update the FILE_SET table */
     proc sql noprint;
        update &dblib..file_set
        set file_set_status_cd='01',
        file_set_member_cnt=&membercount,
        file_set_file_cnt=0,
        file_set_updt_dttm=datetime()
        where file_set_id="&haf_new_fileset_id";
     quit;

   %end;

   %EXIT:

     /* Error handling */
     %if &retcode=1 %then %do; /* error handling */

        /* Update SEA_EXPORT */
        proc sql;
          update &oralib..sea_export 
            set export_dir_path="&haf_output_folder"  
               ,export_status="FAILED"
               ,export_updt_dttm=datetime()
          where export_id="&exportid";
        quit; 

       %if &exporttype=RWE %then %do;
         %rwe_db_connect_info(dblib_ext=rwe_app_lib);

         %let membercount=%sea_utl_get_observation_count(indsn=export.rwe_member);

         /* Update the FILE_SET table */
         proc sql noprint;
            update &dblib..file_set
            set file_set_status_cd='03',
            file_set_updt_dttm=datetime()
            where file_set_id="&haf_new_fileset_id";
         quit;

         /* reset retcode to 1 */
         %let retcode=1;
       %end;

        /*
           Export data set creation ended in error 
           delete any non-meta data sets created
        */
        %if &delete_export=1 %then %do;
           
            proc sql noprint;
               select memname
                 into :tables_to_delete separated by ' '
                 from dictionary.tables
                where libname='EXPORT' and 
                      kscan(memname,1,'_') not in ('META');
            quit;

           %if %SYMEXIST(TABLES_TO_DELETE)>0 %then %do;
             proc datasets lib=EXPORT nolist nowarn;
               delete &tables_to_delete;
             run;
           %end;
                               
        %end;

        /* Error information */
        data error_rpt;
          length info rpt_group $50 value $1000;
          report=1;
          rpt_group='Analytic Job Export Error';
          info='Error Type';
          value="&seaexporterror";
          output;
          info="Export file handling";
        %if %SYMEXIST(FILE_HANDLING) > 0 %then %do;
           value="&file_handling";
        %end;
        run;

        %let sea_status_type=ERROR; 

     %end;  /* end error handling */

     /* Export report */
     %if &exporttype ne RWE %then %do;     
       %sea_export_report;
     %end;

     /* Export email */
     %if %symexist(HAF_EMAIL)> 0 and %nrbquote(&haf_email) ne %str() %then %do; 
       %sea_export_email;
     %end;

     /*  Completion messages  */
     %if &sea_status_type=ERROR | &sea_status_type=FATAL %then
          %sea_utl_add_message(macroName=&currentMacro,msgType=FATAL,
                               msgID=_sea_jobrun_000_error,parm1=&currentMacro);
           %else %sea_utl_add_message(macroName=&currentMacro,msgType=NOTE,
                               msgID=_sea_jobrun_000_note,parm1=&currentMacro);
     
     /*  Write status json  */
     %sea_utl_write_status_json(outDSLibref=EXPORT);

     /* delete the constants file */ 
     proc datasets lib=&clibref nowarn nolist;
       delete &dsn ;
       run;
     quit;   

     /* Delete META_* data sets */
     %if &retcode=0 %then %do;
       proc datasets lib=export nowarn nolist;
         delete meta_ex: ;
         run;
       quit;
     %end;
 
     proc printto log=log;
     run;
  
%mend;             
